;********************************************************************
;                         Tomasz Jaboski                          *
;                 Mikrokontrolery PIC16F8x w praktyce               *
;                      Wydawnictwo BTC, www.btc.pl                  *
;********************************************************************

;program odczytujcy kod przycinitego klawisza. 
;Kod ten jest wywietlany na wywietlaczu LCD (podaczenie
;wywietlacza tak jak na rysunku 8.19).
;po puszczeniu klawisza wywietlany jest kod F0h.
;bad ramki powoduje zwrcenie kodu 00h
#include 16f84.h
#define CLK 	porta,3; linia CLK klawiatury PC
#define DATA 	porta,2; linia DATA klawiatury PC
#define E	porta,1
#define RS	porta,0
kod_klaw0	equ 0ch
kod_klaw1	equ 0dh
licznik		equ 0eh
msek		equ 10h
TMP		equ 11h
adres		equ 12h
		__config 	h'3ff1'

		org 0
	
		bsf RP0
		movlw 1ch
		movwf trisa	;wszystkie porty wyjciowe
		clrf trisb 
		bcf RP0

		call iniLCD
et_wys_kod	movlw 80h
		call wrcmd
		;czekanie na kod klawisza z klawiatury
		call _klaw_pc	;kod klawisza w W i kod_klaw0

		swapf kod_klaw0,w;wywietlenie starszej cyfry hex
		andlw 0fh
		call hexascii
		call wrdata
		movf kod_klaw0,w; wywietlenie mlodszej cyfry hex
		andlw 0fh
		call hexascii
		call wrdata
		goto et_wys_kod

		sleep
;********************************************************************
;konwersja liczby z zakresu 0-F na kod ASCII
;********************************************************************
hexascii	movwf TMP
		movlw .10		
		subwf TMP,w
		btfsc C
		goto _litera
		movf TMP,w
		addlw 30h
		return
_litera		movf TMP,w
		addlw 37h
		return

;********************************************************************
;procedura czekania na przesanie kodu klawisza z klawiatury PC AT
;odczytany kod umieszczony jest w kod_klaw0 i W
;jeeli wystpi bd ramki, to zwarcany jest kod 00h
;********************************************************************
_klaw_pc	clrf kod_klaw0
		clrf kod_klaw1
		movlw .11	;licznik odbieranych bitw
		movwf licznik

et_kl1		btfsc CLK	;czekaj na narastajace zbocze CLK
		goto et_kl1
et_kl2		btfss CLK
		goto et_kl2
		
		bcf C		;C=0 kiedy DATA=0
		btfsc DATA
		bsf C		;C=1 kiedy DATA=1
		rrf kod_klaw1,f
		rrf kod_klaw0,f
		decfsz licznik,f
		goto et_kl1
		
		movlw .5
		movwf licznik	;dosunicie 5 pozycji w prawo
et_kl3		rrf kod_klaw1,f
		rrf kod_klaw0,f
		decfsz licznik,f
		goto et_kl3
		;sprawdzanie ramki i wpisanie do W kodu klawisza
    		btfsc kod_klaw0,0	;czy pierwszy bit jest bitem startu
		goto blad_klaw
		rrf kod_klaw1,f		;usunicie bitu startu
		rrf kod_klaw0,f		;w kod_klaw0 kod klawisza
		btfss kod_klaw1,1	;czy bit stopu
		goto blad_klaw
		movf kod_klaw0,w
		return
blad_klaw	clrw
		return		
		



;--------------------------------------------------------------------
;procedura zapisu danej (RS=1) lub instrukcji (RS=0)
;zapisywana osimiobitowa dana znajduje si w rejestrze W
;--------------------------------------------------------------------
wrdata		bsf RS
		goto et_wr
wrcmd		bcf RS		;RS=0 rejestr instrukcji IR
et_wr		movwf TMP	;instrukcja do TMP
		bsf E		;E=1
		swapf TMP,f	;zamiana polbajtow

		movlw 0f0h	;jedynki na starszym polbajcie
		iorwf TMP,w	;f0h.OR.TMP->W
		movwf portb	;na portb starszy polbajt
		bcf E		;E=0
		nop
		nop
		bsf E		;E=1

		swapf TMP,w	;ponowna zamiana polbajtow i wpisnie do W
		iorlw 0f0h	;f0h.OR.W->W
		movwf portb
		bcf E		;E=0
		btfsc RS	;jaki jest stan linii RS
		goto et_wrdata	;zapis danej
		movlw .2
		call delay_msek	;opznienie 2msek dla instrukcji
		return
et_wrdata	call del_100	;opoznienie 100usek dla kodow znakow
		return
	
;--------------------------------------------------------------------
;porcedura inicjalizacji - interfejs 4bitowy
;--------------------------------------------------------------------
iniLCD		movlw .50	;opoznienie 50msek
		call delay_msek
		
		bcf RS		;RS=0 rejestr IR
		bsf E		;E=1
		movlw .3
		movwf portb

		bcf E		;E=0
		movlw .5	;opoznienie 5msek
		call delay_msek
		bsf E		;E=1
		nop		;ponowne wpisanie 3 modulu
		nop
		bcf E		;E=0
		call del_100	;czekaj 100usek
		bsf E		;E=1
		nop		;ponowne wpisanie 3 modulu
		nop
		bcf E		;E=0
		call del_100	;czekaj 100usek

		movlw 28h	;interfejs 4bitowy, 2linie znaki 5x7
		call wrcmd		
		movlw .8	;display off
		call wrcmd
		movlw .1	;display clear
		call wrcmd
		movlw .6	;entry mod
		call wrcmd
		movlw 0ch	;display on
		call wrcmd	
		return
;--------------------------------------------------------------------
;opnienie ok 100msek
;--------------------------------------------------------------------
del_100		bsf RP0		;bank1	
		movlw .8	;option_reg=8 preskaler do WDT 
		movwf option_reg
		bcf RP0		;bank0
		clrf tmr0	;zerowanie licznika TMR0

et_del100	movlw .50	;50->W (50*2usek=100usek)
		bcf RP0		;bank0
		subwf tmr0,w	;czy W = tmr0
		btfss C
		goto et_del100	;nie rowne
		return

		
;--------------------------------------------------------------------
;opnienie w milisekundach
;--------------------------------------------------------------------		
delay_msek	movwf msek
		bsf RP0		;bank1	
		movlw .1	;option_reg=1 preskaler do TMR0 
		movwf option_reg;podzal preskalera 1:4

et_del1		bcf RP0		;bank0
		clrf tmr0	;zerowanie licznika TMR0

et_del2		movlw .125	;125->W (125*4*2usek=1msek)
		bcf RP0		;bank0
		subwf tmr0,w	;czy W = tmr0
		btfss C
		goto et_del2	;nie rowne

		decfsz msek,f	;modyfikacja licznika milisekund
		goto et_del1
		return


		end
		
